@@ -5,7 +5,7 @@
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://git.sr.ht/~protesilaos/tmr
 ;; Mailing list: https://lists.sr.ht/~protesilaos/tmr
-;; Version: 0.2.2
+;; Version: 0.2.3
 ;; Package-Requires: ((emacs "27.1"))
 ;; Keywords: convenience, timer
 
@@ -44,6 +44,10 @@
 ;; candidates are specified in the user option `tmr-descriptions-list',
 ;; though any arbitrary input is acceptable at the minibuffer prompt.
 ;;
+;; An alternative to the `tmr' command is `tmr-with-description'.  The
+;; difference between the two is that the latter always prompts for a
+;; description.
+;;
 ;; When the timer is set, a message is sent to the echo area recording the
 ;; current time and the point in the future when the timer elapses.  Echo
 ;; area messages can be reviewed with the `view-echo-area-messages' which is
@@ -59,6 +63,9 @@
 ;; Note that it is up to the desktop environment or notification daemon to
 ;; decide how to handle the urgency value.
 ;;
+;; If the `tmr-sound-file' is nil, or the file is not found, no sound will
+;; be played.
+;;
 ;; The `tmr-cancel' command is used to cancel running timers (as set by the
 ;; `tmr' command).  If there is only one timer, it cancels it outright.  If
 ;; there are multiple timers, it produces a minibuffer completion prompt
@@ -71,13 +78,15 @@
 (require 'notifications)
 
 (defgroup tmr ()
-  "TMR Must Recur (super simple timer for my private use)."
+  "TMR May Ring: set timers using a simple notation."
   :group 'data)
 
 (defcustom tmr-sound-file
   "/usr/share/sounds/freedesktop/stereo/alarm-clock-elapsed.oga"
-  "Path to sound file used by `tmr--play-sound'."
-  :type 'file
+  "Path to sound file used by `tmr--play-sound'.
+If nil, don't play any sound."
+  :type '(choice file
+                 (const :tag "Off" nil))
   :group 'tmr)
 
 (defcustom tmr-notification-urgency 'normal
@@ -93,10 +102,19 @@ such notifications."
   :group 'tmr)
 
 (defcustom tmr-descriptions-list (list "Boil water" "Prepare tea" "Bake bread")
-  "Optional description candidates for the current `tmr'."
+  "Optional description candidates for the current `tmr'.
+These are provided as completion candidates when `tmr' is called
+with a DESCRIPTION argument or when `tmr-with-description' is
+used."
   :type '(repeat string)
   :group 'tmr)
 
+(defcustom tmr-notify-function #'tmr-notifications-notify
+  "Function called to send notification.
+It should take two string arguments: the title and the message."
+  :type 'function
+  :group 'tmr)
+
 (defun tmr--unit (time)
   "Determine common time unit for TIME."
   (cond
@@ -123,7 +141,7 @@ such notifications."
 ;; platforms and Emacs needs to be compiled --with-sound capabilities.
 (defun tmr--play-sound ()
   "Play `tmr-sound-file' using the 'ffplay' executable (ffmpeg)."
-  (let ((sound tmr-sound-file))
+  (when-let* ((sound tmr-sound-file))
     (when (file-exists-p sound)
       (unless (executable-find "ffplay")
         (user-error "Cannot play %s without `ffplay'" sound))
@@ -152,8 +170,23 @@ such notifications."
           win))
     (user-error "No *tmr-messages* buffer; have you used `tmr'?")))
 
-(defun tmr--notify-send (start &optional description)
-  "Send system notification for timer with START time.
+(defun tmr-notifications-notify (title message)
+  "Dispatch notification titled TITLE with MESSAGE via D-Bus.
+
+Read: (info \"(elisp) Desktop Notifications\") for details."
+  (notifications-notify
+   :title title
+   :body message
+   :app-name "GNU Emacs"
+   :urgency tmr-notification-urgency
+   :sound-file tmr-sound-file))
+
+(defun tmr--notify-send-notification (title message)
+  "Send notification with TITLE and MESSAGE using `tmr-notify-function'."
+  (funcall tmr-notify-function title message))
+
+(defun tmr--notify (start &optional description)
+  "Send notification for timer with START time.
 Optionally include DESCRIPTION."
   (let ((end (format-time-string "%T"))
         (desc-plain "")
@@ -161,19 +194,16 @@ Optionally include DESCRIPTION."
     (when description
       (setq desc-plain (concat "\n" description)
             desc-propertized (concat " [" (propertize description 'face 'bold) "]")))
-    ;; Read: (info "(elisp) Desktop Notifications")
-    (notifications-notify
-     :title "TMR Must Recur"
-     :body (format "Time is up!\nStarted: %s\nEnded: %s%s"
-                   start end desc-plain)
-     :app-name "GNU Emacs"
-     :urgency tmr-notification-urgency
-     :sound-file tmr-sound-file)
+    (tmr--notify-send-notification
+     "TMR May Ring (Emacs tmr package)"
+     (format "Time is up!\nStarted: %s\nEnded: %s%s"
+             start end desc-plain))
     (message
      "TMR %s %s ; %s %s%s"
      (propertize "Start:" 'face 'success) start
      (propertize "End:" 'face 'error) end
      desc-propertized)
+    (tmr--log-in-buffer (format "Completed at %s what started at %s" end start))
     (unless (plist-get (notifications-get-capabilities) :sound)
       (tmr--play-sound))))
 
@@ -247,9 +277,12 @@ With optional DESCRIPTION as a prefix (\\[universal-argument]),
 prompt for a description among `tmr-descriptions-list', though
 allow for any string to serve as valid input.
 
-This command also plays back `tmr-sound-file'.
+This command also plays back `tmr-sound-file' if it is available.
 
-To cancel the timer, use the `tmr-cancel' command."
+To cancel the timer, use the `tmr-cancel' command.
+
+To always prompt for a DESCRIPTION when setting a timer, use the
+command `tmr-with-description' instead of this one."
   (interactive
    (list
     (read-string "N minutes for timer (append `h' or `s' for other units): ")
@@ -264,9 +297,23 @@ To cancel the timer, use the `tmr-cancel' command."
            object-desc
            (run-with-timer
             unit nil
-            'tmr--notify-send start description))
+            'tmr--notify start description))
           tmr--timers)
     (tmr--log-in-buffer object-desc)))
 
+;;;###autoload
+(defun tmr-with-description (time description)
+  "Set timer to TIME duration and notify with DESCRIPTION after it elapses.
+
+See `tmr' for a description of the arguments.  The difference
+between the two commands is that `tmr-with-description' always
+asks for a description whereas `tmr' only asks for it when the
+user uses a prefix argument (\\[universal-argument])."
+  (interactive
+   (list
+    (read-string "N minutes for timer (append `h' or `s' for other units): ")
+    (tmr--description-prompt)))
+  (tmr time description))
+
 (provide 'tmr)
 ;;; tmr.el ends here
